--[[
HolmerAndRopaPotatoReady

Specialization to allow harvest potato with the Holmer and Ropa vehicles

Author: 	Ifko[nator]
Date: 		18.06.2020
Version:	3.2

History:	v1.0 @15.03.2016 - initial implemation in FS 15
			v2.0 @03.11.2016 - convert to FS 17
			v3.0 @15.05.2020 - convert to FS 19
			v3.1 @04.06.2020 - fix for Error '.../FS19_holmerAndRopaPaotatoReady/HolmerAndRopaPaotatoReady.lua:37: attempt to index field 'fruitTypes' (a nil value)'
			v3.2 @18.06.2020 - add hook for the loadSpecValueFillTypes function from FillUnit.lua to add potatos in the shop view
]]

HolmerAndRopaPotatoReady = {};

function HolmerAndRopaPotatoReady.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(FillUnit, specializations) or SpecializationUtil.hasSpecialization(FruitPreparer, specializations);
end;

function HolmerAndRopaPotatoReady.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",	
		"onUpdateTick"
	};

	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, HolmerAndRopaPotatoReady);
	end;
end;

function HolmerAndRopaPotatoReady:onLoad(savegame)
	local specFillUnit = self.spec_fillUnit;
	local specCutter = self.spec_cutter;

	if specCutter ~= nil then
		if specCutter.fruitTypes ~= nil and specCutter.fruitTypes[FruitType.SUGARBEET] and not specCutter.fruitTypes[FruitType.POTATO] then
			specCutter.fruitTypes[FruitType.POTATO] = true;
		end;
	end;

	if specFillUnit ~= nil then
		local fillUnitConfigurationId = Utils.getNoNil(self.configurations["fillUnit"], 1);
		local baseKey = string.format("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(%d).fillUnits", fillUnitConfigurationId - 1);
		
		local fillUnitNumber = 0;

    	while true do
			local fillUnitKey = string.format("%s.fillUnit(%d)", baseKey, fillUnitNumber);

			if not hasXMLProperty(self.xmlFile, fillUnitKey) then
    	        break;
			end;

			local fillTypes = getXMLString(self.xmlFile, fillUnitKey .. "#fillTypes");
		
			if fillTypes ~= nil then
				if specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes[FillType.SUGARBEET] and not specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes[FillType.POTATO] then
					specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes[FillType.POTATO] = true;
				end;
			end;

			fillUnitNumber = fillUnitNumber + 1;
		end;
	end;
end;

function HolmerAndRopaPotatoReady:onUpdateTick(dt)
	local specFruitPreparer = self.spec_fruitPreparer;
	
	if specFruitPreparer == nil or self.getCombine == nil then
		return;
	else
		local combine = self:getCombine();
		
		if combine ~= nil and combine:getIsTurnedOn() then		
			local specCutter = self.spec_cutter;
			local specFillUnit = combine.spec_fillUnit;
			local specCombine = combine.spec_combine;

			if specFillUnit.fillUnits ~= nil then	
				for _, fillUnit in pairs(specFillUnit.fillUnits) do
					if fillUnit.fillType == FillType.UNKNOWN then
						if specFruitPreparer.fruitType == FruitType.POTATO then
							local sugarBeet = g_fruitTypeManager:getFruitTypeByName("sugarBeet");
							
							if sugarBeet ~= nil then
								specFruitPreparer.fruitType = sugarBeet.index;
								
								if self.setAIFruitRequirements ~= nil then
               						self:setAIFruitRequirements(sugarBeet.index, sugarBeet.minPreparingGrowthState, sugarBeet.maxPreparingGrowthState);
									   
									local aiUsePreparedState = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.fruitPreparer#aiUsePreparedState"), specCutter ~= nil);
									   
									if aiUsePreparedState then
               						    self:addAIFruitRequirement(sugarBeet.index, sugarBeet.preparedGrowthState, sugarBeet.preparedGrowthState);
               						end;
								end;

								combine.fruitPreparerFruitType = sugarBeet.index;
							end;
						elseif specFruitPreparer.fruitType == FruitType.SUGARBEET then
							local potato = g_fruitTypeManager:getFruitTypeByName("potato");
				
							if potato ~= nil then
								specFruitPreparer.fruitType = potato.index;
								
								if self.setAIFruitRequirements ~= nil then
               						self:setAIFruitRequirements(potato.index, potato.minPreparingGrowthState, potato.maxPreparingGrowthState);
									   
									local aiUsePreparedState = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.fruitPreparer#aiUsePreparedState"), specCutter ~= nil);
									   
									if aiUsePreparedState then
               						    self:addAIFruitRequirement(potato.index, potato.preparedGrowthState, potato.preparedGrowthState);
               						end;
								end;

								combine.fruitPreparerFruitType = potato.index;
							end;
						end;
					elseif specFruitPreparer.currentInputFruitType ~= specFruitPreparer.fruitType then	
						local currentFruitType = specFruitPreparer.currentInputFruitType;
						
						if combine ~= nil then	
							currentFruitType = g_fruitTypeManager:getFruitTypeByName(g_fillTypeManager:getFillTypeByIndex(fillUnit.fillType).name);
						end;
						
						if currentFruitType ~= nil then
							specFruitPreparer.fruitType = currentFruitType.index;
							
							if self.setAIFruitRequirements ~= nil then
               					self:setAIFruitRequirements(currentFruitType.index, currentFruitType.minPreparingGrowthState, currentFruitType.maxPreparingGrowthState);
								   
								local aiUsePreparedState = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.fruitPreparer#aiUsePreparedState"), specCutter ~= nil);
								   
								if aiUsePreparedState then
               					    self:addAIFruitRequirement(currentFruitType.index, currentFruitType.preparedGrowthState, currentFruitType.preparedGrowthState);
               					end;
							end;
							
							combine.fruitPreparerFruitType = currentFruitType.index;

							specFruitPreparer.currentInputFruitType = specFruitPreparer.fruitType;
						end;
					end;
				end;
			end;
		end;
	end;
end;

--## Hook loadSpecValueFillTypes function from FillUnit.lua to add potatos in the shop view
FillUnit.loadSpecValueFillTypes = function(xmlFile, customEnvironment)
	--print("xmlFile = " .. tostring(xmlFile) .. " customEnvironment = " .. tostring(customEnvironment));
	
	local fillTypeNames, fillTypeCategoryNames, fillTypes, fruitTypeNames = nil;
	local rootName = getXMLRootName(xmlFile);
	local fillUnitConfigurationNumber = 0;

	while true do
		local fillUnitConfigurationKey = string.format(rootName .. ".fillUnit.fillUnitConfigurations.fillUnitConfiguration(%d)", fillUnitConfigurationNumber);

		if not hasXMLProperty(xmlFile, fillUnitConfigurationKey) then
			break;
		end;

		local fillUnitNumber = 0;

		while true do
			local fillUnitKey = string.format(fillUnitConfigurationKey .. ".fillUnits.fillUnit(%d)", fillUnitNumber);

			if not hasXMLProperty(xmlFile, fillUnitKey) then
				break;
			end;

			local showInShop = getXMLBool(xmlFile, fillUnitKey .. "#showInShop");
			local capacity = getXMLFloat(xmlFile, fillUnitKey .. "#capacity");

			if (showInShop == nil or showInShop) and (capacity == nil or capacity > 0) then
				local currentFillTypes = getXMLString(xmlFile, fillUnitKey .. "#fillTypes");

				if currentFillTypes ~= nil then
					if string.find(currentFillTypes, "sugarBeet") and not string.find(currentFillTypes, "potato") then
						currentFillTypes = currentFillTypes .. " potato"; --## add potato to the shop view
					end;
					
					if fillTypeNames == nil then
						fillTypeNames = currentFillTypes;
					else
						fillTypeNames = fillTypeNames .. " " .. currentFillTypes;
					end;
				end;

				local currentFillTypeCategories = getXMLString(xmlFile, fillUnitKey .. "#fillTypeCategories");

				if currentFillTypeCategories ~= nil then
					if fillTypeCategoryNames == nil then
						fillTypeCategoryNames = currentFillTypeCategories;
					else
						fillTypeCategoryNames = fillTypeCategoryNames .. " " .. currentFillTypeCategories;
					end;
				end;
			end;

			fillUnitNumber = fillUnitNumber + 1;
		end;

		fillUnitConfigurationNumber = fillUnitConfigurationNumber + 1;
	end;

	if fillTypeNames == nil then
		fillTypeNames = getXMLString(xmlFile, rootName .. ".fillTypes");
	end;

	fillTypeNames = Utils.getNoNil(getXMLString(xmlFile, rootName .. ".storeData.specs.fillTypes"), fillTypeNames);

	if fillTypeNames ~= nil then
		fillTypeCategoryNames = nil;
	end;

	if fillTypeCategoryNames == nil then
		fillTypeCategoryNames = getXMLString(xmlFile, rootName .. ".fillTypeCategories");
	end;

	if fillTypes == nil then
		fruitTypeNames = getXMLString(xmlFile, rootName .. ".fruitTypes");
	end;

	if fillTypes == nil then
		fruitTypeNames = getXMLString(xmlFile, rootName .. ".cutter#fruitTypes");

		if fruitTypeNames ~= nil then
			if string.find(fruitTypeNames, "sugarBeet") and not string.find(fruitTypeNames, "potato") then
				fruitTypeNames = fruitTypeNames .. " potato"; --## add potato to the shop view
			end;
		end;
	end;

	local fruitTypeCategoryNames = getXMLString(xmlFile, rootName .. ".cutter#fruitTypeCategories");
	local useWindrowed = Utils.getNoNil(getXMLBool(xmlFile, rootName .. ".cutter#useWindrowed"), false);

	fillTypeCategoryNames = Utils.getNoNil(getXMLString(xmlFile, rootName .. ".storeData.specs.fillTypeCategories"), fillTypeCategoryNames);

	return {
		categoryNames = fillTypeCategoryNames,
		fillTypeNames = fillTypeNames,
		fruitTypeNames = fruitTypeNames,
		fruitTypeCategoryNames = fruitTypeCategoryNames,
		useWindrowed = useWindrowed
	};
end;